@@ -38,6 +38,8 @@ class QWidget;
 class KServicePrivate;
 
 /**
+ * @class KService kservice.h <KService>
+ *
  * Represent a service, like an application or plugin
  * bound to one or several mimetypes (or servicetypes) as written
  * in its desktop entry file.
@@ -124,6 +126,12 @@ public:
      *         or QString() if not set
      */
     QString terminalOptions() const;
+    /**
+     * Checks whether the service runs on a discrete graphics card
+     * @return true if the service has to run under a discrete graphics card
+     * @since 5.30
+     */
+    bool runOnDiscreteGpu() const;
     /**
      * Checks whether the service runs with a different user id.
      * @return true if the service has to be run under a different uid.
@@ -479,8 +487,8 @@ public:
      * @return The path to use for the new KService.
      */
     static QString newServicePath(bool showInMenu, const QString &suggestedName,
-                                  QString *menuId = 0,
-                                  const QStringList *reservedMenuIds = 0);
+                                  QString *menuId = nullptr,
+                                  const QStringList *reservedMenuIds = nullptr);
 
     /**
      * This template allows to load the library for the specified service and ask the
@@ -495,8 +503,8 @@ public:
      *         factory was unable to create an object of the given type.
      */
     template <class T>
-    T *createInstance(QObject *parent = 0,
-                      const QVariantList &args = QVariantList(), QString *error = 0) const
+    T *createInstance(QObject *parent = nullptr,
+                      const QVariantList &args = QVariantList(), QString *error = nullptr) const
     {
         return createInstance<T>(0, parent, args, error);
     }
@@ -516,7 +524,7 @@ public:
      */
     template <class T>
     T *createInstance(QWidget *parentWidget, QObject *parent,
-                      const QVariantList &args = QVariantList(), QString *error = 0) const
+                      const QVariantList &args = QVariantList(), QString *error = nullptr) const
     {
         KPluginLoader pluginLoader(*this);
         KPluginFactory *factory = pluginLoader.factory();
