@@ -51,19 +51,19 @@
 #include <qstandardpaths.h>
 #include <QLockFile>
 
-static const char *s_cSycocaPath = 0;
+static const char *s_cSycocaPath = nullptr;
 
 KBuildSycocaInterface::~KBuildSycocaInterface() {}
 
 KBuildSycoca::KBuildSycoca(bool globalDatabase)
     : KSycoca(true),
-      m_allEntries(0),
-      m_currentFactory(0),
-      m_ctimeFactory(0),
-      m_ctimeDict(0),
-      m_currentEntryDict(0),
-      m_serviceGroupEntryDict(0),
-      m_vfolder(0),
+      m_allEntries(nullptr),
+      m_currentFactory(nullptr),
+      m_ctimeFactory(nullptr),
+      m_ctimeDict(nullptr),
+      m_currentEntryDict(nullptr),
+      m_serviceGroupEntryDict(nullptr),
+      m_vfolder(nullptr),
       m_newTimestamp(0),
       m_globalDatabase(globalDatabase),
       m_menuTest(false),
@@ -88,8 +88,8 @@ KSycocaEntry::Ptr KBuildSycoca::createEntry(const QString &file, bool addToFacto
     if (m_allEntries) {
         Q_ASSERT(m_ctimeDict);
         quint32 oldTimestamp = m_ctimeDict->ctime(file, m_resource);
-        if (file.contains("fake")) {
-            qDebug() << "m_ctimeDict->ctime(" << file << ") = " << oldTimestamp << "compared with" << timeStamp;
+        if (file.contains(QLatin1String("fake"))) {
+            qCDebug(SYCOCA) << "m_ctimeDict->ctime(" << file << ") = " << oldTimestamp << "compared with" << timeStamp;
         }
 
         if (timeStamp && (timeStamp == oldTimestamp)) {
@@ -102,17 +102,17 @@ KSycocaEntry::Ptr KBuildSycoca::createEntry(const QString &file, bool addToFacto
             // remove from m_ctimeDict; if m_ctimeDict is not empty
             // after all files have been processed, it means
             // some files were removed since last time
-            if (file.contains("fake")) {
-                qDebug() << "reusing (and removing) old entry for:" << file << "entry=" << entry;
+            if (file.contains(QLatin1String("fake"))) {
+                qCDebug(SYCOCA) << "reusing (and removing) old entry for:" << file << "entry=" << entry;
             }
             m_ctimeDict->remove(file, m_resource);
         } else if (oldTimestamp) {
             m_changed = true;
             m_ctimeDict->remove(file, m_resource);
-            qDebug() << "modified:" << file;
+            qCDebug(SYCOCA) << "modified:" << file;
         } else {
             m_changed = true;
-            qDebug() << "new:" << file;
+            qCDebug(SYCOCA) << "new:" << file;
         }
     }
     m_ctimeFactory->dict()->addCTime(file, m_resource, timeStamp);
@@ -140,28 +140,26 @@ KService::Ptr KBuildSycoca::createService(const QString &path)
 // returns false if the database is up to date, true if it needs to be saved
 bool KBuildSycoca::build()
 {
-    typedef QLinkedList<KBSEntryDict *> KBSEntryDictList;
+    typedef QList<KBSEntryDict *> KBSEntryDictList;
     KBSEntryDictList entryDictList;
-    KBSEntryDict *serviceEntryDict = 0;
+    KBSEntryDict *serviceEntryDict = nullptr;
 
     // Convert for each factory the entryList to a Dict.
+    entryDictList.reserve(factories()->size());
     int i = 0;
     // For each factory
-    for (KSycocaFactoryList::Iterator factory = factories()->begin();
-            factory != factories()->end();
-            ++factory) {
+    Q_FOREACH (KSycocaFactory* factory, *factories()) {
         KBSEntryDict *entryDict = new KBSEntryDict;
         if (m_allEntries) { // incremental build
-            const KSycocaEntry::List list = (*m_allEntries)[i++];
-            Q_FOREACH (const KSycocaEntry::Ptr &entry, list) {
+            Q_FOREACH (const KSycocaEntry::Ptr &entry, (*m_allEntries)[i++]) {
                 //if (entry->entryPath().contains("fake"))
-                //    qDebug() << "inserting into entryDict:" << entry->entryPath() << entry;
+                //    qCDebug(SYCOCA) << "inserting into entryDict:" << entry->entryPath() << entry;
                 entryDict->insert(entry->entryPath(), entry);
             }
         }
-        if ((*factory) == d->m_serviceFactory) {
+        if (factory == d->m_serviceFactory) {
             serviceEntryDict = entryDict;
-        } else if ((*factory) == m_buildServiceGroupFactory) {
+        } else if (factory == m_buildServiceGroupFactory) {
             m_serviceGroupEntryDict = entryDict;
         }
         entryDictList.append(entryDict);
@@ -180,11 +178,9 @@ bool KBuildSycoca::build()
 
     QMap<QString, QByteArray> allResourcesSubDirs; // dirs, kstandarddirs-resource-name
     // For each factory
-    for (KSycocaFactoryList::Iterator factory = factories()->begin();
-            factory != factories()->end();
-            ++factory) {
+    Q_FOREACH (KSycocaFactory* factory, *factories()) {
         // For each resource the factory deals with
-        const KSycocaResourceList *list = (*factory)->resourceList();
+        const KSycocaResourceList *list = factory->resourceList();
         if (!list) {
             continue;
         }
@@ -197,7 +193,6 @@ bool KBuildSycoca::build()
     }
 
     m_ctimeFactory = new KCTimeFactory(this); // This is a build factory too, don't delete!!
-    bool uptodate = true;
     for (QMap<QString, QByteArray>::ConstIterator it1 = allResourcesSubDirs.constBegin();
             it1 != allResourcesSubDirs.constEnd();
             ++it1) {
@@ -205,7 +200,7 @@ bool KBuildSycoca::build()
         m_resourceSubdir = it1.key();
         m_resource = it1.value();
 
-        QStringList relFiles;
+        QSet<QString> relFiles;
         const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, m_resourceSubdir, QStandardPaths::LocateDirectory);
         Q_FOREACH (const QString &dir, dirs) {
             QDirIterator it(dir, QDirIterator::Subdirectories);
@@ -213,68 +208,62 @@ bool KBuildSycoca::build()
                 const QString filePath = it.next();
                 Q_ASSERT(filePath.startsWith(dir)); // due to the line below...
                 const QString relPath = filePath.mid(dir.length() + 1);
-                if (!relFiles.contains(relPath)) {
-                    relFiles.append(relPath);
-                }
+                relFiles.insert(relPath);
             }
         }
         // Now find all factories that use this resource....
         // For each factory
-        KBSEntryDictList::const_iterator ed_it = entryDictList.begin();
-        const KBSEntryDictList::const_iterator ed_end = entryDictList.end();
+        KBSEntryDictList::const_iterator ed_it = entryDictList.constBegin();
+        const KBSEntryDictList::const_iterator ed_end = entryDictList.constEnd();
         KSycocaFactoryList::const_iterator it = factories()->constBegin();
         const KSycocaFactoryList::const_iterator end = factories()->constEnd();
         for (; it != end; ++it, ++ed_it) {
             m_currentFactory = (*it);
             // m_ctimeInfo gets created after the initial loop, so it has no entryDict.
-            m_currentEntryDict = ed_it == ed_end ? 0 : *ed_it;
+            m_currentEntryDict = ed_it == ed_end ? nullptr : *ed_it;
             // For each resource the factory deals with
             const KSycocaResourceList *list = m_currentFactory->resourceList();
             if (!list) {
                 continue;
             }
 
-            for (KSycocaResourceList::ConstIterator it2 = list->constBegin();
-                    it2 != list->constEnd();
-                    ++it2) {
-                KSycocaResource res = (*it2);
+            Q_FOREACH (const KSycocaResource &res, *list) {
                 if (res.resource != (*it1)) {
                     continue;
                 }
 
                 // For each file in the resource
-                for (QStringList::ConstIterator it3 = relFiles.constBegin();
-                        it3 != relFiles.constEnd();
-                        ++it3) {
+                for (auto entryPath = relFiles.constBegin();
+                         entryPath != relFiles.constEnd();
+                        ++entryPath) {
                     // Check if file matches filter
-                    if ((*it3).endsWith(res.extension)) {
-                        QString entryPath = (*it3);
-                        createEntry(entryPath, true);
+                    if ((*entryPath).endsWith(res.extension)) {
+                        createEntry(*entryPath, true);
                     }
                 }
             }
         }
         if (m_changed || !m_allEntries) {
-            uptodate = false;
-            //qDebug() << "CHANGED:" << resource;
+            //qCDebug(SYCOCA) << "CHANGED:" << m_resource;
             m_changedResources.append(m_resource);
         }
     }
 
-    bool result = !uptodate || (m_ctimeDict && !m_ctimeDict->isEmpty());
     if (m_ctimeDict && !m_ctimeDict->isEmpty()) {
-        //qDebug() << "Still in time dict:";
+        //qCDebug(SYCOCA) << "Still in time dict:";
         //m_ctimeDict->dump();
         // ## It seems entries filtered out by vfolder are still in there,
-        // so we end up always saving ksycoca, i.e. this method never returns false
+        // so on a real system we end up always adding "apps" to m_changedResources
 
         // Get the list of resources from which some files were deleted
         const QStringList resources = m_ctimeDict->remainingResourceList();
-        qDebug() << "Still in the time dict (i.e. deleted files)" << resources;
+        qCDebug(SYCOCA) << "Still in the time dict (i.e. deleted files)" << resources;
         m_changedResources += resources;
     }
 
-    if (result || m_menuTest) {
+    bool result = true;
+    const bool createVFolder = !m_changedResources.isEmpty() || (m_ctimeDict && !m_ctimeDict->isEmpty());
+    if (createVFolder || m_menuTest) {
         m_resource = "apps";
         m_resourceSubdir = QStringLiteral("applications");
         m_currentFactory = d->m_serviceFactory;
@@ -309,8 +298,7 @@ bool KBuildSycoca::build()
         }
 
         if (m_changed || !m_allEntries) {
-            uptodate = false;
-            //qDebug() << "CHANGED:" << m_resource;
+            //qCDebug(SYCOCA) << "CHANGED:" << m_resource;
             m_changedResources.append(m_resource);
         }
         if (m_menuTest) {
@@ -347,7 +335,7 @@ void KBuildSycoca::createMenu(const QString &caption_, const QString &name_, VFo
                 if (group) {
                     entry = KServiceGroup::Ptr(static_cast<KServiceGroup*>(group.data()));
                     if (entry->directoryEntryPath() != directoryFile) {
-                        entry = 0;    // Can't reuse this one!
+                        entry = nullptr;    // Can't reuse this one!
                     }
                 }
             }
@@ -388,15 +376,15 @@ bool KBuildSycoca::recreate(bool incremental)
     }
     QString path(fi.absoluteFilePath());
 
-    QLockFile lockFile(path + ".lock");
+    QLockFile lockFile(path + QLatin1String(".lock"));
     if (!lockFile.tryLock()) {
-        qDebug() <<  "Waiting for already running" << KBUILDSYCOCA_EXENAME << "to finish.";
+        qCDebug(SYCOCA) <<  "Waiting for already running" << KBUILDSYCOCA_EXENAME << "to finish.";
         if (!lockFile.lock()) {
             qCWarning(SYCOCA) << "Couldn't lock" << path + ".lock";
             return false;
         }
         if (!needsRebuild()) {
-            //qDebug() << "Up-to-date, skipping.";
+            //qCDebug(SYCOCA) << "Up-to-date, skipping.";
             return true;
         }
     }
@@ -404,10 +392,10 @@ bool KBuildSycoca::recreate(bool incremental)
     QByteArray qSycocaPath = QFile::encodeName(path);
     s_cSycocaPath = qSycocaPath.data();
 
-    m_allEntries = 0;
-    m_ctimeDict = 0;
+    m_allEntries = nullptr;
+    m_ctimeDict = nullptr;
     if (incremental && checkGlobalHeader()) {
-        qDebug() << "Reusing existing ksycoca";
+        qCDebug(SYCOCA) << "Reusing existing ksycoca";
         KSycoca *oldSycoca = KSycoca::self();
         m_allEntries = new KSycocaEntryListList;
         m_ctimeDict = new KCTimeDict;
@@ -421,7 +409,7 @@ bool KBuildSycoca::recreate(bool incremental)
         KCTimeFactory *ctimeInfo = new KCTimeFactory(oldSycoca);
         *m_ctimeDict = ctimeInfo->loadDict();
     }
-    s_cSycocaPath = 0;
+    s_cSycocaPath = nullptr;
 
     QSaveFile database(path);
     bool openedOK = database.open(QIODevice::WriteOnly);
@@ -439,7 +427,7 @@ bool KBuildSycoca::recreate(bool incremental)
     str->setVersion(QDataStream::Qt_5_3);
 
     m_newTimestamp = QDateTime::currentMSecsSinceEpoch();
-    qDebug().nospace() << "Recreating ksycoca file (" << path << ", version " << KSycoca::version() << ")";
+    qCDebug(SYCOCA).nospace() << "Recreating ksycoca file (" << path << ", version " << KSycoca::version() << ")";
 
     // It is very important to build the servicetype one first
     KBuildServiceTypeFactory *buildServiceTypeFactory = new KBuildServiceTypeFactory(this);
@@ -456,7 +444,7 @@ bool KBuildSycoca::recreate(bool incremental)
             database.cancelWriting();    // Error
         }
         delete str;
-        str = 0;
+        str = nullptr;
 
         //if we are currently via sudo, preserve the original owner
         //as $HOME may also be that of another user rather than /root
@@ -491,12 +479,12 @@ bool KBuildSycoca::recreate(bool incremental)
         }
     } else {
         delete str;
-        str = 0;
+        str = nullptr;
         database.cancelWriting();
         if (m_menuTest) {
             return true;
         }
-        qDebug() << "Database is up to date";
+        qCDebug(SYCOCA) << "Database is up to date";
     }
 
     if (m_globalDatabase) {
@@ -525,21 +513,19 @@ void KBuildSycoca::save(QDataStream *str)
     (*str) << qint32(KSycoca::version());
     //KSycocaFactory * servicetypeFactory = 0;
     //KBuildMimeTypeFactory * mimeTypeFactory = 0;
-    KBuildServiceFactory *serviceFactory = 0;
-    for (KSycocaFactoryList::Iterator factory = factories()->begin();
-            factory != factories()->end();
-            ++factory) {
+    KBuildServiceFactory *serviceFactory = nullptr;
+    Q_FOREACH (KSycocaFactory* factory, *factories()) {
         qint32 aId;
         qint32 aOffset;
-        aId = (*factory)->factoryId();
+        aId = factory->factoryId();
         //if ( aId == KST_KServiceTypeFactory )
-        //   servicetypeFactory = *factory;
+        //   servicetypeFactory = factory;
         //else if ( aId == KST_KMimeTypeFactory )
-        //   mimeTypeFactory = static_cast<KBuildMimeTypeFactory *>( *factory );
+        //   mimeTypeFactory = static_cast<KBuildMimeTypeFactory *>( factory );
         if (aId == KST_KServiceFactory) {
-            serviceFactory = static_cast<KBuildServiceFactory *>(*factory);
+            serviceFactory = static_cast<KBuildServiceFactory *>(factory);
         }
-        aOffset = (*factory)->offset(); // not set yet, so always 0
+        aOffset = factory->offset(); // not set yet, so always 0
         (*str) << aId;
         (*str) << aOffset;
     }
@@ -559,13 +545,11 @@ void KBuildSycoca::save(QDataStream *str)
     if (serviceFactory) serviceFactory->postProcessServices();
 
     // Here so that it's the last debug message
-    qDebug() << "Saving";
+    qCDebug(SYCOCA) << "Saving";
 
     // Write factory data....
-    for (KSycocaFactoryList::Iterator factory = factories()->begin();
-            factory != factories()->end();
-            ++factory) {
-        (*factory)->save(*str);
+    Q_FOREACH (KSycocaFactory* factory, *factories()) {
+        factory->save(*str);
         if (str->status() != QDataStream::Ok) { // ######## TODO: does this detect write errors, e.g. disk full?
             return;    // error
         }
@@ -577,12 +561,11 @@ void KBuildSycoca::save(QDataStream *str)
     str->device()->seek(0);
 
     (*str) << qint32(KSycoca::version());
-    for (KSycocaFactoryList::Iterator factory = factories()->begin();
-            factory != factories()->end(); ++factory) {
+    Q_FOREACH (KSycocaFactory* factory, *factories()) {
         qint32 aId;
         qint32 aOffset;
-        aId = (*factory)->factoryId();
-        aOffset = (*factory)->offset();
+        aId = factory->factoryId();
+        aOffset = factory->offset();
         (*str) << aId;
         (*str) << aOffset;
     }
@@ -594,8 +577,8 @@ void KBuildSycoca::save(QDataStream *str)
 
 QStringList KBuildSycoca::factoryResourceDirs()
 {
-    static QStringList *dirs = NULL;
-    if (dirs != NULL) {
+    static QStringList *dirs = nullptr;
+    if (dirs != nullptr) {
         return *dirs;
     }
     dirs = new QStringList;
@@ -609,8 +592,8 @@ QStringList KBuildSycoca::factoryResourceDirs()
 
 QStringList KBuildSycoca::existingResourceDirs()
 {
-    static QStringList *dirs = NULL;
-    if (dirs != NULL) {
+    static QStringList *dirs = nullptr;
+    if (dirs != nullptr) {
         return *dirs;
     }
     dirs = new QStringList(factoryResourceDirs());
